@@ -21,6 +21,8 @@
 
 #include <string.h>
 
+#include <libtracker-miners-common/tracker-debug.h>
+
 #include "tracker-decorator.h"
 #include "tracker-priority-queue.h"
 #include "tracker-decorator-private.h"
@@ -56,11 +58,6 @@ struct _TrackerDecoratorInfo {
 	gint ref_count;
 };
 
-struct _ClassInfo {
-	gchar *class_name;
-	gint priority;
-};
-
 struct _SparqlUpdate {
 	gchar *sparql;
 	gint id;
@@ -69,7 +66,6 @@ struct _SparqlUpdate {
 struct _TrackerDecoratorPrivate {
 	TrackerNotifier *notifier;
 
-	GArray *classes; /* Array of ClassInfo */
 	gchar **class_names;
 
 	gssize n_remaining_items;
@@ -77,6 +73,8 @@ struct _TrackerDecoratorPrivate {
 
 	GQueue item_cache; /* Queue of TrackerDecoratorInfo */
 
+	GStrv priority_graphs;
+
 	/* Arrays of tracker IDs */
 	GArray *prepended_ids;
 	GSequence *blocklist_items;
@@ -102,7 +100,6 @@ struct _TrackerDecoratorPrivate {
 enum {
 	PROP_CLASS_NAMES = 1,
 	PROP_COMMIT_BATCH_SIZE,
-	PROP_PRIORITY_RDF_TYPES,
 };
 
 enum {
@@ -580,10 +577,13 @@ decorator_cancel_active_tasks (TrackerDecorator *decorator)
 	g_hash_table_remove_all (priv->tasks);
 }
 
-static gchar *
-create_query_string (TrackerDecorator  *decorator,
-                     gchar            **select_clauses)
+static gboolean
+append_graph_patterns (TrackerDecorator *decorator,
+                       GString          *query,
+                       gboolean          priority,
+                       gboolean          first)
 {
+	TrackerDecoratorPrivate *priv = decorator->priv;
 	const gchar *graphs[] = {
 		"tracker:Audio",
 		"tracker:Pictures",
@@ -591,7 +591,32 @@ create_query_string (TrackerDecorator  *decorator,
 		"tracker:Software",
 		"tracker:Documents",
 	};
+	gint i;
+
+	for (i = 0; i < G_N_ELEMENTS (graphs); i++) {
+		if (priority !=
+		    (priv->priority_graphs &&
+		     g_strv_contains ((const gchar * const *) priv->priority_graphs, graphs[i])))
+			continue;
+
+		if (!first)
+			g_string_append (query, "UNION ");
+
+		g_string_append_printf (query,
+		                        "{ GRAPH %s { ?urn a nfo:FileDataObject } } ",
+		                        graphs[i]);
+		first = FALSE;
+	}
+
+	return first;
+}
+
+static gchar *
+create_query_string (TrackerDecorator  *decorator,
+                     gchar            **select_clauses)
+{
 	GString *query;
+	gboolean first;
 	gint i;
 
 	query = g_string_new ("SELECT ");
@@ -602,15 +627,9 @@ create_query_string (TrackerDecorator  *decorator,
 
 	g_string_append (query, "{ ");
 
-	for (i = 0; i < G_N_ELEMENTS (graphs); i++) {
-		if (i > 0) {
-			g_string_append (query, "UNION ");
-		}
-
-		g_string_append_printf (query,
-		                        "{ GRAPH %s { ?urn a nfo:FileDataObject } } ",
-		                        graphs[i]);
-	}
+	/* Add priority graphs first, so they come up first in the query */
+	first = append_graph_patterns (decorator, query, TRUE, TRUE);
+	append_graph_patterns (decorator, query, FALSE, first);
 
 	g_string_append_printf (query,
 	                        "FILTER (NOT EXISTS {"
@@ -668,7 +687,7 @@ ensure_remaining_items_query (TrackerDecorator *decorator)
 }
 
 static void
-decorator_query_remaining_items_cb (GObject      *object,
+decorator_count_remaining_items_cb (GObject      *object,
                                     GAsyncResult *result,
                                     gpointer      user_data)
 {
@@ -693,7 +712,7 @@ decorator_query_remaining_items_cb (GObject      *object,
 		tracker_sparql_cursor_get_integer (cursor, 0);
 	g_object_unref (cursor);
 
-	g_debug ("Found %" G_GSIZE_FORMAT " items to extract", priv->n_remaining_items);
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Found %" G_GSIZE_FORMAT " items to extract", priv->n_remaining_items));
 
 	if (priv->n_remaining_items > 0)
 		decorator_cache_next_items (decorator);
@@ -702,7 +721,7 @@ decorator_query_remaining_items_cb (GObject      *object,
 }
 
 static void
-decorator_query_remaining_items (TrackerDecorator *decorator)
+decorator_count_remaining_items (TrackerDecorator *decorator)
 {
 	gchar *clauses[] = { "COUNT(?urn)", NULL };
 	TrackerDecoratorPrivate *priv;
@@ -717,7 +736,7 @@ decorator_query_remaining_items (TrackerDecorator *decorator)
 		                                   "offset", 0);
 		tracker_sparql_statement_execute_async (priv->item_count_query,
 		                                        priv->cancellable,
-		                                        decorator_query_remaining_items_cb,
+		                                        decorator_count_remaining_items_cb,
 		                                        decorator);
 	} else {
 		decorator_notify_empty (decorator);
@@ -819,7 +838,8 @@ decorator_cache_next_items (TrackerDecorator *decorator)
         priv->querying = TRUE;
 
 	if (priv->n_remaining_items == 0) {
-		decorator_query_remaining_items (decorator);
+		TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Counting items which still need processing"));
+		decorator_count_remaining_items (decorator);
 	} else {
 		TrackerSparqlStatement *statement;
 		gint offset;
@@ -830,6 +850,7 @@ decorator_cache_next_items (TrackerDecorator *decorator)
 		if (priv->commit_buffer)
 			offset += priv->commit_buffer->len;
 
+		TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Querying items which still need processing"));
 		statement = ensure_remaining_items_query (decorator);
 		tracker_sparql_statement_bind_int (statement, "offset", offset);
 		tracker_sparql_statement_execute_async (statement,
@@ -861,38 +882,6 @@ tracker_decorator_get_property (GObject    *object,
 	}
 }
 
-static void
-decorator_add_class (TrackerDecorator *decorator,
-                     const gchar      *class)
-{
-	TrackerDecoratorPrivate *priv = decorator->priv;
-	ClassInfo info;
-
-	info.class_name = g_strdup (class);
-	info.priority = G_PRIORITY_DEFAULT;
-	g_array_append_val (priv->classes, info);
-}
-
-static void
-decorator_set_classes (TrackerDecorator  *decorator,
-                       const gchar      **classes)
-{
-	TrackerDecoratorPrivate *priv = decorator->priv;
-	gint i;
-
-	g_strfreev (priv->class_names);
-	priv->class_names = g_strdupv ((gchar **) classes);
-
-	if (priv->classes->len > 0) {
-		g_array_remove_range (priv->classes, 0,
-		                      priv->classes->len);
-	}
-
-	for (i = 0; classes[i]; i++) {
-		decorator_add_class (decorator, classes[i]);
-	}
-}
-
 static void
 tracker_decorator_set_property (GObject      *object,
                                 guint         param_id,
@@ -905,16 +894,9 @@ tracker_decorator_set_property (GObject      *object,
 	priv = decorator->priv;
 
 	switch (param_id) {
-	case PROP_CLASS_NAMES:
-		decorator_set_classes (decorator, g_value_get_boxed (value));
-		break;
 	case PROP_COMMIT_BATCH_SIZE:
 		priv->batch_size = g_value_get_int (value);
 		break;
-	case PROP_PRIORITY_RDF_TYPES:
-		tracker_decorator_set_priority_rdf_types (decorator,
-		                                          g_value_get_boxed (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 	}
@@ -1003,6 +985,7 @@ tracker_decorator_finalize (GObject *object)
 
 	g_clear_object (&priv->remaining_items_query);
 	g_clear_object (&priv->item_count_query);
+	g_strfreev (priv->priority_graphs);
 
 	g_cancellable_cancel (priv->cancellable);
 	g_clear_object (&priv->cancellable);
@@ -1019,7 +1002,6 @@ tracker_decorator_finalize (GObject *object)
 
 	g_strfreev (priv->class_names);
 	g_hash_table_destroy (priv->tasks);
-	g_array_unref (priv->classes);
 	g_array_unref (priv->prepended_ids);
 	g_clear_pointer (&priv->sparql_buffer, g_array_unref);
 	g_clear_pointer (&priv->commit_buffer, g_array_unref);
@@ -1034,6 +1016,7 @@ tracker_decorator_paused (TrackerMiner *miner)
 {
 	TrackerDecoratorPrivate *priv;
 
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Paused"));
 	decorator_cancel_active_tasks (TRACKER_DECORATOR (miner));
 	priv = TRACKER_DECORATOR (miner)->priv;
 	g_timer_stop (priv->timer);
@@ -1044,6 +1027,7 @@ tracker_decorator_resumed (TrackerMiner *miner)
 {
 	TrackerDecoratorPrivate *priv;
 
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Resumed"));
 	decorator_cache_next_items (TRACKER_DECORATOR (miner));
 	priv = TRACKER_DECORATOR (miner)->priv;
 	g_timer_continue (priv->timer);
@@ -1054,6 +1038,7 @@ tracker_decorator_stopped (TrackerMiner *miner)
 {
 	TrackerDecoratorPrivate *priv;
 
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Stopped"));
 	decorator_cancel_active_tasks (TRACKER_DECORATOR (miner));
 	priv = TRACKER_DECORATOR (miner)->priv;
 	g_timer_stop (priv->timer);
@@ -1068,6 +1053,7 @@ tracker_decorator_started (TrackerMiner *miner)
 	decorator = TRACKER_DECORATOR (miner);
 	priv = decorator->priv;
 
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Started"));
 	g_timer_start (priv->timer);
 	decorator_rebuild_cache (decorator);
 }
@@ -1103,14 +1089,6 @@ tracker_decorator_class_init (TrackerDecoratorClass *klass)
 	                                                   0, G_MAXINT, DEFAULT_BATCH_SIZE,
 	                                                   G_PARAM_READWRITE |
 	                                                   G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (object_class,
-	                                 PROP_PRIORITY_RDF_TYPES,
-	                                 g_param_spec_boxed ("priority-rdf-types",
-	                                                     "Priority RDF types",
-	                                                     "rdf:type that needs to be extracted first",
-	                                                     G_TYPE_STRV,
-	                                                     G_PARAM_WRITABLE |
-	                                                     G_PARAM_STATIC_STRINGS));
 	/**
 	 * TrackerDecorator::items-available:
 	 * @decorator: the #TrackerDecorator
@@ -1148,20 +1126,12 @@ tracker_decorator_class_init (TrackerDecoratorClass *klass)
 		              G_TYPE_NONE, 0);
 }
 
-static void
-class_info_clear (ClassInfo *info)
-{
-	g_free (info->class_name);
-}
-
 static void
 tracker_decorator_init (TrackerDecorator *decorator)
 {
 	TrackerDecoratorPrivate *priv;
 
 	decorator->priv = priv = tracker_decorator_get_instance_private (decorator);
-	priv->classes = g_array_new (FALSE, FALSE, sizeof (ClassInfo));
-	g_array_set_clear_func (priv->classes, (GDestroyNotify) class_info_clear);
 	priv->blocklist_items = g_sequence_new (NULL);
 	priv->prepended_ids = g_array_new (FALSE, FALSE, sizeof (gint));
 	priv->batch_size = DEFAULT_BATCH_SIZE;
@@ -1245,6 +1215,7 @@ tracker_decorator_prepend_id (TrackerDecorator *decorator,
 	g_array_append_val (priv->prepended_ids, id);
 
 	/* The resource was explicitly requested, remove it from blocklists */
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Removing id %i from blocklist", id));
 	decorator_blocklist_remove (decorator, id);
 }
 
@@ -1278,6 +1249,7 @@ tracker_decorator_delete_id (TrackerDecorator *decorator,
 	}
 
 	/* Blocklist the item so it's not processed in the future */
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Added id %i to blocklist", id));
 	decorator_blocklist_add (decorator, id);
 }
 
@@ -1290,7 +1262,7 @@ tracker_decorator_delete_id (TrackerDecorator *decorator,
  *
  * Processes the next resource in the queue to have extended metadata
  * extracted. If the item in the queue has been completed already, it
- * signals it's completion instead.
+ * signals its completion instead.
  *
  * This function will give a #GError if the miner is paused at the
  * time it is called.
@@ -1323,6 +1295,7 @@ tracker_decorator_next (TrackerDecorator    *decorator,
 		return;
 	}
 
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Queued task %s", g_task_get_name (task)));
 	g_queue_push_tail (&priv->next_elem_queue, task);
 	decorator_pair_tasks (decorator);
 }
@@ -1354,63 +1327,14 @@ tracker_decorator_next_finish (TrackerDecorator  *decorator,
 	return g_task_propagate_pointer (G_TASK (result), error);
 }
 
-static gint
-class_compare_func (const ClassInfo *a,
-                    const ClassInfo *b)
-{
-	return b->priority - a->priority;
-}
-
-static void
-decorator_set_class_priority (TrackerDecorator *decorator,
-                              const gchar      *class,
-                              gint              priority)
+void
+tracker_decorator_set_priority_graphs (TrackerDecorator    *decorator,
+                                       const gchar * const *graphs)
 {
 	TrackerDecoratorPrivate *priv = decorator->priv;
-	ClassInfo *info;
-	gint i;
 
-	for (i = 0; i < priv->classes->len; i++) {
-		info = &g_array_index (priv->classes, ClassInfo, i);
-
-		if (strcmp (info->class_name, class) != 0)
-			continue;
-
-		info->priority = priority;
-		break;
-	}
-}
-
-/**
- * tracker_decorator_set_priority_rdf_types:
- * @decorator: a #TrackerDecorator
- * @rdf_types: a string array of rdf types
- *
- * Re-evaluate the priority queues internally to ensure that
- * @rdf_types are handled before all other content. This is useful for
- * applications that need their content available sooner than the
- * standard time it would take to index content.
- *
- * Since: 0.18
- **/
-void
-tracker_decorator_set_priority_rdf_types (TrackerDecorator    *decorator,
-                                          const gchar * const *rdf_types)
-{
-	TrackerDecoratorPrivate *priv;
-	gint i;
-
-	g_return_if_fail (TRACKER_DECORATOR (decorator));
-	g_return_if_fail (rdf_types != NULL);
-
-	priv = decorator->priv;
-
-	for (i = 0; rdf_types[i]; i++) {
-		decorator_set_class_priority (decorator, rdf_types[i],
-		                              G_PRIORITY_HIGH);
-	}
-
-	g_array_sort (priv->classes, (GCompareFunc) class_compare_func);
+	g_strfreev (priv->priority_graphs);
+	priv->priority_graphs = g_strdupv ((gchar **) graphs);
 	decorator_rebuild_cache (decorator);
 }
 
@@ -1508,6 +1432,7 @@ void
 tracker_decorator_info_complete (TrackerDecoratorInfo *info,
                                  gchar                *sparql)
 {
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Task for %s completed successfully", info->url));
 	g_task_return_pointer (info->task, sparql, g_free);
 }
 
@@ -1525,6 +1450,7 @@ void
 tracker_decorator_info_complete_error (TrackerDecoratorInfo *info,
                                        GError               *error)
 {
+	TRACKER_NOTE (DECORATOR, g_message ("[Decorator] Task for %s failed: %s", info->url, error->message));
 	g_task_return_error (info->task, error);
 }
 
