@@ -417,9 +417,6 @@ sparql_buffer_push_to_pool (TrackerSparqlBuffer *buffer,
 
 	if (tracker_task_pool_limit_reached (TRACKER_TASK_POOL (buffer))) {
 		tracker_sparql_buffer_flush (buffer, "SPARQL buffer limit reached");
-	} else if (priv->tasks->len > tracker_task_pool_get_limit (TRACKER_TASK_POOL (buffer)) / 2) {
-		/* We've filled half of the buffer, flush it as we receive more tasks */
-		tracker_sparql_buffer_flush (buffer, "SPARQL buffer half-full");
 	}
 }
 
@@ -519,3 +516,36 @@ tracker_sparql_buffer_push_finish (TrackerSparqlBuffer  *buffer,
 
 	return task;
 }
+
+static gboolean
+task_has_file (TrackerTask *task,
+               GFile       *file)
+{
+	GFile *task_file;
+
+	task_file = tracker_task_get_file (task);
+
+	return g_file_equal (task_file, file);
+}
+
+TrackerSparqlBufferState
+tracker_sparql_buffer_get_state (TrackerSparqlBuffer *buffer,
+                                 GFile               *file)
+{
+	TrackerSparqlBufferPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_SPARQL_BUFFER (buffer), TRACKER_BUFFER_STATE_UNKNOWN);
+	g_return_val_if_fail (G_IS_FILE (file), TRACKER_BUFFER_STATE_UNKNOWN);
+
+	priv = tracker_sparql_buffer_get_instance_private (TRACKER_SPARQL_BUFFER (buffer));
+
+	if (!tracker_task_pool_find (TRACKER_TASK_POOL (buffer), file))
+		return TRACKER_BUFFER_STATE_UNKNOWN;
+
+	if (priv->tasks &&
+	    g_ptr_array_find_with_equal_func (priv->tasks, file,
+	                                      (GEqualFunc) task_has_file, NULL))
+		return TRACKER_BUFFER_STATE_QUEUED;
+
+	return TRACKER_BUFFER_STATE_FLUSHING;
+}
