@@ -38,7 +38,6 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-#include "tracker-log.h"
 #include "tracker-file-utils.h"
 #include "tracker-type-utils.h"
 
@@ -253,7 +252,7 @@ tracker_file_get_mime_type (GFile *file)
 
 #define __bsize f_frsize
 
-#if HAVE_STATVFS64
+#ifdef HAVE_STATVFS64
 #define __statvfs statvfs64
 #else
 #define __statvfs statvfs
