@@ -99,15 +99,15 @@ extract_watchdog_start (TrackerExtractWatchdog *watchdog,
 	const gchar *domain_name = watchdog->domain;
 	gchar *tracker_extract_dbus_name;
 
-	g_debug ("Setting up watch on tracker-extract (autostart: %s)",
-		 autostart ? "yes" : "no");
-
 	if (domain_name == NULL) {
 		tracker_extract_dbus_name = g_strdup (TRACKER_MINER_DBUS_NAME_PREFIX "Extract");
 	} else {
 		tracker_extract_dbus_name = g_strconcat (domain_name, ".Tracker3.Miner.Extract", NULL);
 	}
 
+	g_debug ("Setting up watch on tracker-extract at %s (autostart: %s)",
+		 tracker_extract_dbus_name, autostart ? "yes" : "no");
+
 	watchdog->extractor_watchdog_id =
 		g_bus_watch_name (TRACKER_IPC_BUS,
 				  tracker_extract_dbus_name,
