@@ -37,7 +37,6 @@
 
 #include "tracker-config.h"
 #include "tracker-miner-files.h"
-#include "tracker-miner-files-index.h"
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -56,6 +55,8 @@
 #define LOCALE_FILENAME "locale-for-miner-apps.txt"
 
 static GMainLoop *main_loop;
+static GDBusProxy *index_proxy;
+static GPtrArray *proxy_folders;
 
 static gint initial_sleep = -1;
 static gboolean no_daemon;
@@ -64,6 +65,10 @@ static gboolean version;
 static guint miners_timeout_id = 0;
 static gboolean do_crawling = FALSE;
 static gchar *domain_ontology_name = NULL;
+static gboolean dry_run = FALSE;
+
+static gboolean slept = TRUE;
+static gboolean graphs_ready = FALSE;
 
 static GOptionEntry entries[] = {
 	{ "initial-sleep", 's', 0,
@@ -83,6 +88,10 @@ static GOptionEntry entries[] = {
 	  G_OPTION_ARG_STRING, &domain_ontology_name,
 	  N_("Runs for a specific domain ontology"),
 	  NULL },
+	{ "dry-run", 'r', 0,
+	  G_OPTION_ARG_NONE, &dry_run,
+	  N_("Avoids changes in the filesystem"),
+	  NULL },
 	{ "version", 'V', 0,
 	  G_OPTION_ARG_NONE, &version,
 	  N_("Displays version information"),
@@ -368,13 +377,23 @@ miner_do_start (TrackerMiner *miner)
 	}
 }
 
+static void
+miner_maybe_start (TrackerMiner *miner)
+{
+	if (!slept || !graphs_ready)
+		return;
+
+	miner_do_start (miner);
+}
+
 static gboolean
 miner_start_idle_cb (gpointer data)
 {
 	TrackerMiner *miner = data;
 
 	miners_timeout_id = 0;
-	miner_do_start (miner);
+	slept = TRUE;
+	miner_maybe_start (miner);
 	return G_SOURCE_REMOVE;
 }
 
@@ -387,13 +406,13 @@ miner_start (TrackerMiner  *miner,
 
 	if (!do_mtime_checking) {
 		g_debug ("Avoiding initial sleep, no mtime check needed");
-		miner_do_start (miner);
+		miner_maybe_start (miner);
 		return;
 	}
 
 	/* If requesting to run as no-daemon, start right away */
 	if (no_daemon) {
-		miner_do_start (miner);
+		miner_maybe_start (miner);
 		return;
 	}
 
@@ -401,10 +420,11 @@ miner_start (TrackerMiner  *miner,
 	initial_sleep = tracker_config_get_initial_sleep (config);
 
 	if (initial_sleep <= 0) {
-		miner_do_start (miner);
+		miner_maybe_start (miner);
 		return;
 	}
 
+	slept = FALSE;
 	g_message ("Performing initial sleep of %d seconds",
 	           initial_sleep);
 	miners_timeout_id = g_timeout_add_seconds (initial_sleep,
@@ -426,7 +446,7 @@ miner_finished_cb (TrackerMinerFS *fs,
 	        total_directories_found,
 	        total_files_found);
 
-	if (do_crawling) {
+	if (do_crawling && !dry_run) {
 		tracker_miner_files_set_last_crawl_done (TRACKER_MINER_FILES (fs),
 							 TRUE);
 	}
@@ -435,8 +455,8 @@ miner_finished_cb (TrackerMinerFS *fs,
 	 * the mainloop and exit.
 	 */
 	if (no_daemon && main_loop) {
-		// FIXME: wait for extractor to finish
-/*		g_main_loop_quit (main_loop);*/
+		/* FIXME: wait for extractor to finish */
+		g_main_loop_quit (main_loop);
 	}
 }
 
@@ -477,6 +497,19 @@ dummy_log_handler (const gchar    *domain,
 	return;
 }
 
+static void
+graphs_created_cb (GObject      *source,
+                   GAsyncResult *res,
+                   gpointer      user_data)
+{
+	TrackerMiner *miner = user_data;
+
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (source),
+	                                         res, NULL);
+	graphs_ready = TRUE;
+	miner_maybe_start (miner);
+}
+
 static void
 check_eligible (void)
 {
@@ -754,16 +787,17 @@ setup_connection_and_endpoint (TrackerDomainOntology    *domain,
                                TrackerEndpointDBus     **endpoint,
                                GError                  **error)
 {
-	GFile *store, *ontology;
+	GFile *store = NULL, *ontology;
 
-	store = get_cache_dir (domain);
+	if (!dry_run)
+		store = get_cache_dir (domain);
 	ontology = tracker_domain_ontology_get_ontology (domain);
 	*sparql_conn = tracker_sparql_connection_new (get_fts_connection_flags (),
 	                                              store,
 	                                              ontology,
 	                                              NULL,
 	                                              error);
-	g_object_unref (store);
+	g_clear_object (&store);
 
 	if (!*sparql_conn)
 		return FALSE;
@@ -779,12 +813,120 @@ setup_connection_and_endpoint (TrackerDomainOntology    *domain,
 	return TRUE;
 }
 
+static void
+update_indexed_files_from_proxy (TrackerMinerFiles *miner,
+                                 GDBusProxy        *proxy)
+{
+	TrackerIndexingTree *indexing_tree;
+	const gchar **indexed_uris = NULL;
+	GVariant *v;
+	gint i;
+
+	v = g_dbus_proxy_get_cached_property (proxy, "IndexedLocations");
+	if (v)
+		indexed_uris = g_variant_get_strv (v, NULL);
+
+	indexing_tree = tracker_miner_fs_get_indexing_tree (TRACKER_MINER_FS (miner));
+
+	/* Remove folders no longer there */
+	for (i = 0; i < proxy_folders->len; i++) {
+		GFile *file;
+		gchar *uri;
+
+		file = g_ptr_array_index (proxy_folders, i);
+		uri = g_file_get_uri (file);
+
+		if (!indexed_uris || !g_strv_contains (indexed_uris, uri)) {
+			tracker_indexing_tree_remove (indexing_tree,
+			                              file);
+		}
+
+		g_free (uri);
+	}
+
+	for (i = 0; indexed_uris && indexed_uris[i]; i++) {
+		GFileInfo *file_info;
+		GFile *file;
+
+		file = g_file_new_for_uri (indexed_uris[i]);
+		if (g_ptr_array_find_with_equal_func (proxy_folders,
+		                                      file,
+		                                      (GEqualFunc) g_file_equal,
+		                                      NULL)) {
+			g_object_unref (file);
+			continue;
+		}
+
+		file_info = g_file_query_info (file,
+		                               G_FILE_ATTRIBUTE_STANDARD_TYPE,
+		                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+		                               NULL, NULL);
+
+		if (!file_info) {
+			g_object_unref (file);
+			continue;
+		}
+
+		if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
+			if (!tracker_indexing_tree_file_is_indexable (indexing_tree,
+			                                              file,
+			                                              G_FILE_TYPE_DIRECTORY)) {
+				tracker_indexing_tree_add (indexing_tree,
+				                           file,
+				                           TRACKER_DIRECTORY_FLAG_RECURSE |
+				                           TRACKER_DIRECTORY_FLAG_CHECK_MTIME |
+				                           TRACKER_DIRECTORY_FLAG_MONITOR);
+				g_ptr_array_add (proxy_folders, g_object_ref (file));
+			} else {
+				tracker_indexing_tree_notify_update (indexing_tree,
+				                                     file, TRUE);
+			}
+		} else {
+			tracker_miner_fs_check_file (TRACKER_MINER_FS (miner),
+			                             file, G_PRIORITY_HIGH, TRUE);
+		}
+
+		g_object_unref (file_info);
+		g_object_unref (file);
+	}
+
+	g_free (indexed_uris);
+}
+
+static void
+proxy_properties_changed_cb (GDBusProxy *proxy,
+                             GVariant   *changed_properties,
+                             GStrv       invalidated_properties,
+                             gpointer    user_data)
+{
+	update_indexed_files_from_proxy (user_data, proxy);
+}
+
+static void
+on_tracker_index_proxy_ready (GObject      *source,
+                              GAsyncResult *res,
+                              gpointer      user_data)
+{
+	TrackerMinerFiles *miner = user_data;
+	GError *error = NULL;
+
+	index_proxy = g_dbus_proxy_new_finish (res, &error);
+	if (error) {
+		g_critical ("Could not set up proxy: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	g_signal_connect (index_proxy, "g-properties-changed",
+	                  G_CALLBACK (proxy_properties_changed_cb), miner);
+	update_indexed_files_from_proxy (miner, index_proxy);
+}
+
 int
 main (gint argc, gchar *argv[])
 {
 	TrackerConfig *config;
 	TrackerMiner *miner_files;
-	TrackerMinerFilesIndex *miner_files_index;
 	GOptionContext *context;
 	GError *error = NULL;
 	gboolean do_mtime_checking;
@@ -794,6 +936,7 @@ main (gint argc, gchar *argv[])
 	TrackerSparqlConnection *sparql_conn;
 	TrackerEndpointDBus *endpoint;
 	TrackerDomainOntology *domain_ontology;
+	GCancellable *cancellable;
 	gchar *domain_name, *dbus_name;
 
 	main_loop = NULL;
@@ -904,6 +1047,18 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
+	proxy_folders = g_ptr_array_new_with_free_func (g_object_unref);
+	cancellable = g_cancellable_new ();
+	g_dbus_proxy_new (connection,
+	                  G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
+	                  NULL,
+	                  "org.freedesktop.Tracker3.Miner.Files.Control",
+	                  "/org/freedesktop/Tracker3/Miner/Files/Proxy",
+	                  "org.freedesktop.Tracker3.Miner.Files.Proxy",
+	                  cancellable,
+	                  on_tracker_index_proxy_ready,
+	                  miner_files);
+
 	/* If the locales changed, we need to reset some things first */
 	detect_locale_changed (TRACKER_MINER (miner_files), domain_ontology);
 
@@ -916,14 +1071,6 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-	/* Create new TrackerMinerFilesIndex object */
-	miner_files_index = tracker_miner_files_index_new (TRACKER_MINER_FILES (miner_files));
-	if (!miner_files_index) {
-		g_object_unref (miner_files);
-		g_object_unref (config);
-		return EXIT_FAILURE;
-	}
-
 	/* Request DBus name */
 	dbus_name = tracker_domain_ontology_get_domain (domain_ontology, DBUS_NAME_SUFFIX);
 
@@ -966,14 +1113,25 @@ main (gint argc, gchar *argv[])
 	 * event of a crash, this is changed back on shutdown if
 	 * everything appears to be fine.
 	 */
-	tracker_miner_files_set_need_mtime_check (TRACKER_MINER_FILES (miner_files), TRUE);
+	if (!dry_run) {
+		tracker_miner_files_set_need_mtime_check (TRACKER_MINER_FILES (miner_files), TRUE);
+		tracker_miner_files_set_mtime_checking (TRACKER_MINER_FILES (miner_files), do_mtime_checking);
+	}
 
-	/* Configure files miner */
-	tracker_miner_files_set_mtime_checking (TRACKER_MINER_FILES (miner_files), do_mtime_checking);
 	g_signal_connect (miner_files, "finished",
 			  G_CALLBACK (miner_finished_cb),
 			  NULL);
 
+	/* Preempt creation of graphs */
+	tracker_sparql_connection_update_async (tracker_miner_get_connection (miner_files),
+	                                        "CREATE SILENT GRAPH tracker:FileSystem; "
+	                                        "CREATE SILENT GRAPH tracker:Software; "
+	                                        "CREATE SILENT GRAPH tracker:Documents; "
+	                                        "CREATE SILENT GRAPH tracker:Pictures; "
+	                                        "CREATE SILENT GRAPH tracker:Audio; "
+	                                        "CREATE SILENT GRAPH tracker:Video ",
+	                                        NULL, graphs_created_cb, miner_files);
+
 	if (do_crawling)
 		miner_start (miner_files, config, do_mtime_checking);
 
@@ -984,14 +1142,18 @@ main (gint argc, gchar *argv[])
 
 	g_debug ("Shutdown started");
 
-	if (miners_timeout_id == 0 && !miner_needs_check (miner_files)) {
+	if (!dry_run && miners_timeout_id == 0 && !miner_needs_check (miner_files)) {
 		tracker_miner_files_set_need_mtime_check (TRACKER_MINER_FILES (miner_files), FALSE);
 		save_current_locale (domain_ontology);
 	}
 
 	g_main_loop_unref (main_loop);
 	g_object_unref (config);
-	g_object_unref (miner_files_index);
+
+	g_cancellable_cancel (cancellable);
+	g_object_unref (cancellable);
+	g_clear_object (&index_proxy);
+	g_clear_pointer (&proxy_folders, g_ptr_array_unref);
 
 	g_object_unref (miner_files);
 
@@ -999,6 +1161,9 @@ main (gint argc, gchar *argv[])
 	g_object_unref (connection);
 	tracker_domain_ontology_unref (domain_ontology);
 
+	tracker_sparql_connection_close (sparql_conn);
+	g_object_unref (sparql_conn);
+
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
