@@ -18,7 +18,7 @@
  * Boston, MA  02110-1301, USA.
  */
 
-#include <config.h>
+#include "config-miners.h"
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -182,8 +182,6 @@ extract_ps (const gchar *uri)
 
 #ifdef USING_UNZIPPSFILES
 
-#include <zlib.h>
-
 static TrackerResource *
 extract_ps_gz (const gchar *uri)
 {
